@@ -20,6 +20,8 @@ static int	compare_pairs(_pdfio_pair_t *a, _pdfio_pair_t *b);
 //
 // 'pdfioDictClear()' - Remove a key/value pair from a dictionary.
 //
+// @since PDFio v1.4@
+//
 
 bool					// O - `true` if cleared, `false` otherwise
 pdfioDictClear(pdfio_dict_t *dict,	// I - Dictionary
@@ -30,7 +32,7 @@ pdfioDictClear(pdfio_dict_t *dict,	// I - Dictionary
 		pkey;			// Search key
 
 
-  PDFIO_DEBUG("pdfioDictClear(dict=%p, key=\"%s\")\n", dict, key);
+  PDFIO_DEBUG("pdfioDictClear(dict=%p, key=\"%s\")\n", (void *)dict, key);
 
   if (!dict || !key)
     return (false);
@@ -75,7 +77,7 @@ pdfioDictCopy(pdfio_file_t *pdf,	// I - PDF file
   _pdfio_value_t	v;		// Current destination value
 
 
-  PDFIO_DEBUG("pdfioDictCopy(pdf=%p, dict=%p(%p))\n", pdf, dict, dict ? dict->pdf : NULL);
+  PDFIO_DEBUG("pdfioDictCopy(pdf=%p, dict=%p(%p))\n", (void *)pdf, (void *)dict, dict ? (void *)dict->pdf : NULL);
 
   // Create the new dictionary...
   if ((ndict = pdfioDictCreate(pdf)) == NULL)
@@ -345,6 +347,8 @@ pdfioDictGetDict(pdfio_dict_t *dict,	// I - Dictionary
 //
 // 'pdfioDictGetKey()' - Get the key for the specified pair.
 //
+// @since PDFio v1.4@
+//
 
 const char *				// O - Key for specified pair
 pdfioDictGetKey(pdfio_dict_t *dict,	// I - Dictionary
@@ -375,6 +379,8 @@ pdfioDictGetName(pdfio_dict_t *dict,	// I - Dictionary
 //
 // 'pdfioDictGetNumPairs()' - Get the number of key/value pairs in a dictionary.
 //
+// @since PDFio v1.4@
+//
 
 size_t					// O - Number of pairs
 pdfioDictGetNumPairs(pdfio_dict_t *dict)// I - Dictionary
@@ -458,134 +464,25 @@ pdfioDictGetString(pdfio_dict_t *dict,	// I - Dictionary
   _pdfio_value_t *value = _pdfioDictGetValue(dict, key);
 
 
+  PDFIO_DEBUG("pdfioDictGetString(dict=%p, key=\"%s\")\n", (void *)dict, key);
+  PDFIO_DEBUG("pdfioDictGetString: value=%p(type=%d)\n", (void *)value, value ? value->type : 0);
+
   if (value && value->type == PDFIO_VALTYPE_STRING)
   {
+    PDFIO_DEBUG("pdfioDictGetString: Returning \"%s\".\n", value->value.string);
     return (value->value.string);
   }
   else if (value && value->type == PDFIO_VALTYPE_BINARY && value->value.binary.datalen < 4096)
   {
     // Convert binary string to regular string...
-    char	temp[4096],		// Temporary string
-		*tempptr;		// Pointer into temporary string
-    unsigned char *dataptr;		// Pointer into the data string
+    char	temp[4096];		// Temporary UTF-8 string
 
-    if (!(value->value.binary.datalen & 1) && !memcmp(value->value.binary.data, "\376\377", 2))
+    if (!(value->value.binary.datalen & 1) && (!memcmp(value->value.binary.data, "\376\377", 2) || !memcmp(value->value.binary.data, "\377\376", 2)))
     {
-      // Copy UTF-16 BE
-      int	ch;			// Unicode character
-      size_t	remaining;		// Remaining bytes
+      // Copy UTF-16...
+      PDFIO_DEBUG("pdfioDictGetString: Converting UTF-16 to UTF-8 string.\n");
 
-      for (dataptr = value->value.binary.data + 2, remaining = value->value.binary.datalen - 2, tempptr = temp; remaining > 1 && tempptr < (temp + sizeof(temp) - 5); dataptr += 2, remaining -= 2)
-      {
-        ch = (dataptr[0] << 8) | dataptr[1];
-
-        if (ch >= 0xd800 && ch <= 0xdbff && remaining > 3)
-	{
-	  // Multi-word UTF-16 char...
-          int lch;			// Lower bits
-
-	  lch = (dataptr[2] << 8) | dataptr[3];
-
-          if (lch < 0xdc00 || lch >= 0xdfff)
-	    break;
-
-          ch        = (((ch & 0x3ff) << 10) | (lch & 0x3ff)) + 0x10000;
-          dataptr   += 2;
-          remaining -= 2;
-	}
-	else if (ch >= 0xfffe)
-	{
-	  continue;
-	}
-
-        if (ch < 128)
-        {
-          // ASCII
-          *tempptr++ = (char)ch;
-        }
-        else if (ch < 4096)
-        {
-          // 2-byte UTF-8
-          *tempptr++ = (char)(0xc0 | (ch >> 6));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-        else if (ch < 65536)
-        {
-          // 3-byte UTF-8
-          *tempptr++ = (char)(0xe0 | (ch >> 12));
-          *tempptr++ = (char)(0x80 | ((ch >> 6) & 0x3f));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-        else
-        {
-          // 4-byte UTF-8
-          *tempptr++ = (char)(0xe0 | (ch >> 18));
-          *tempptr++ = (char)(0x80 | ((ch >> 12) & 0x3f));
-          *tempptr++ = (char)(0x80 | ((ch >> 6) & 0x3f));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-      }
-
-      *tempptr = '\0';
-    }
-    else if (!(value->value.binary.datalen & 1) && !memcmp(value->value.binary.data, "\377\376", 2))
-    {
-      // Copy UTF-16 LE
-      int	ch;			// Unicode character
-      size_t	remaining;		// Remaining bytes
-
-      for (dataptr = value->value.binary.data + 2, remaining = value->value.binary.datalen - 2, tempptr = temp; remaining > 1 && tempptr < (temp + sizeof(temp) - 5); dataptr += 2, remaining -= 2)
-      {
-        ch = (dataptr[1] << 8) | dataptr[0];
-
-        if (ch >= 0xd800 && ch <= 0xdbff && remaining > 3)
-	{
-	  // Multi-word UTF-16 char...
-          int lch;			// Lower bits
-
-	  lch = (dataptr[3] << 8) | dataptr[2];
-
-          if (lch < 0xdc00 || lch >= 0xdfff)
-	    break;
-
-          ch        = (((ch & 0x3ff) << 10) | (lch & 0x3ff)) + 0x10000;
-          dataptr   += 2;
-          remaining -= 2;
-	}
-	else if (ch >= 0xfffe)
-	{
-	  continue;
-	}
-
-        if (ch < 128)
-        {
-          // ASCII
-          *tempptr++ = (char)ch;
-        }
-        else if (ch < 4096)
-        {
-          // 2-byte UTF-8
-          *tempptr++ = (char)(0xc0 | (ch >> 6));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-        else if (ch < 65536)
-        {
-          // 3-byte UTF-8
-          *tempptr++ = (char)(0xe0 | (ch >> 12));
-          *tempptr++ = (char)(0x80 | ((ch >> 6) & 0x3f));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-        else
-        {
-          // 4-byte UTF-8
-          *tempptr++ = (char)(0xe0 | (ch >> 18));
-          *tempptr++ = (char)(0x80 | ((ch >> 12) & 0x3f));
-          *tempptr++ = (char)(0x80 | ((ch >> 6) & 0x3f));
-          *tempptr++ = (char)(0x80 | (ch & 0x3f));
-        }
-      }
-
-      *tempptr = '\0';
+      _pdfio_utf16cpy(temp, value->value.binary.data, value->value.binary.datalen, sizeof(temp));
     }
     else
     {
@@ -598,10 +495,13 @@ pdfioDictGetString(pdfio_dict_t *dict,	// I - Dictionary
     value->type         = PDFIO_VALTYPE_STRING;
     value->value.string = pdfioStringCreate(dict->pdf, temp);
 
+    PDFIO_DEBUG("pdfioDictGetString: Returning \"%s\".\n", value->value.string);
+
     return (value->value.string);
   }
   else
   {
+    PDFIO_DEBUG("pdfioDictGetString: Returning NULL.\n");
     return (NULL);
   }
 }
@@ -634,7 +534,7 @@ _pdfioDictGetValue(pdfio_dict_t *dict,	// I - Dictionary
 		*match;			// Matching key pair
 
 
-  PDFIO_DEBUG("_pdfioDictGetValue(dict=%p, key=\"%s\")\n", dict, key);
+  PDFIO_DEBUG("_pdfioDictGetValue(dict=%p, key=\"%s\")\n", (void *)dict, key);
 
   if (!dict || !dict->num_pairs || !key)
   {
@@ -677,6 +577,8 @@ _pdfioDictGetValue(pdfio_dict_t *dict,	// I - Dictionary
 // The iteration continues as long as the callback returns `true` or all keys
 // have been iterated.
 //
+// @since PDFio v1.1@
+//
 
 void
 pdfioDictIterateKeys(
@@ -717,7 +619,7 @@ _pdfioDictRead(pdfio_file_t   *pdf,	// I - PDF file
   _pdfio_value_t	value;		// Dictionary value
 
 
-  PDFIO_DEBUG("_pdfioDictRead(pdf=%p, obj=%p, tb=%p, depth=%lu)\n", pdf, obj, tb, (unsigned long)depth);
+  PDFIO_DEBUG("_pdfioDictRead(pdf=%p, obj=%p, tb=%p, depth=%lu)\n", (void *)pdf, (void *)obj, (void *)tb, (unsigned long)depth);
 
   // Create a dictionary and start reading...
   if ((dict = pdfioDictCreate(pdf)) == NULL)
@@ -737,11 +639,6 @@ _pdfioDictRead(pdfio_file_t   *pdf,	// I - PDF file
       _pdfioFileError(pdf, "Invalid dictionary contents.");
       break;
     }
-    else if (_pdfioDictGetValue(dict, key + 1))
-    {
-      _pdfioFileError(pdf, "Duplicate dictionary key '%s'.", key + 1);
-      return (NULL);
-    }
 
     // Then get the next value...
     PDFIO_DEBUG("_pdfioDictRead: Reading value for '%s'.\n", key + 1);
@@ -751,8 +648,17 @@ _pdfioDictRead(pdfio_file_t   *pdf,	// I - PDF file
       _pdfioFileError(pdf, "Missing value for dictionary key '%s'.", key + 1);
       break;
     }
-
-    if (!_pdfioDictSetValue(dict, pdfioStringCreate(pdf, key + 1), &value))
+    else if (_pdfioDictGetValue(dict, key + 1))
+    {
+      // Issue 118: Discard duplicate key/value pairs, in the future this will
+      // be a warning message...
+      _pdfioValueDelete(&value);
+      if (_pdfioFileError(pdf, "WARNING: Discarding value for duplicate dictionary key '%s'.", key + 1))
+        continue;
+      else
+        break;
+    }
+    else if (!_pdfioDictSetValue(dict, pdfioStringCreate(pdf, key + 1), &value))
       break;
 
     PDFIO_DEBUG("_pdfioDictRead: Set %s.\n", key);
@@ -1087,7 +993,7 @@ _pdfioDictSetValue(
   _pdfio_pair_t	*pair;			// Current pair
 
 
-  PDFIO_DEBUG("_pdfioDictSetValue(dict=%p, key=\"%s\", value=%p)\n", dict, key, (void *)value);
+  PDFIO_DEBUG("_pdfioDictSetValue(dict=%p, key=\"%s\", value=%p)\n", (void *)dict, key, (void *)value);
 
   // See if the key is already set...
   if (dict->num_pairs > 0)
