@@ -10,7 +10,7 @@
 #ifndef PDFIO_PRIVATE_H
 #  define PDFIO_PRIVATE_H
 #  ifdef _WIN32
-#    define _CRT_SECURE_NO_WARNINGS	// Disable bogus VS warnings/errors...
+#    define _CRT_SECURE_NO_WARNINGS 1	// Disable bogus VS warnings/errors...
 #  endif // _WIN32
 #  include "pdfio.h"
 #  include <stdarg.h>
@@ -28,16 +28,16 @@
 #    define access	_access		// Map standard POSIX/C99 names
 #    define close	_close
 #    define fileno	_fileno
-#    define lseek	_lseek
+#    define lseek(f,o,w) (off_t)_lseek((f),(long)(o),(w))
 #    define mkdir(d,p)	_mkdir(d)
 #    define open	_open
-#    define read	_read
+#    define read(f,b,s)	_read((f),(b),(unsigned)(s))
 #    define rmdir	_rmdir
 #    define snprintf	_snprintf
 #    define strdup	_strdup
 #    define unlink	_unlink
 #    define vsnprintf	_vsnprintf
-#    define write	_write
+#    define write(f,b,s) _write((f),(b),(unsigned)(s))
 #    ifndef F_OK
 #      define F_OK	00		// POSIX parameters/flags
 #      define W_OK	02
@@ -94,6 +94,7 @@
 //
 
 #  define PDFIO_MAX_DEPTH	32	// Maximum nesting depth for values
+#  define PDFIO_MAX_STRING	65536	// Maximum length of string
 
 typedef void (*_pdfio_extfree_t)(void *);
 					// Extension data free function
@@ -116,6 +117,21 @@ typedef enum _pdfio_predictor_e		// PNG predictor constants
   _PDFIO_PREDICTOR_PNG_AUTO = 15	// PNG "auto" predictor (currently mapped to Paeth)
 } _pdfio_predictor_t;
 
+typedef enum _pdfio_profile_e		// PDF profile constants
+{
+  _PDFIO_PROFILE_NONE = 0,		// Base PDF file
+  _PDFIO_PROFILE_PCLM,			// PCLm (PDF raster) file
+  _PDFIO_PROFILE_PDFA_1A,		// PDF/A-1a:2005
+  _PDFIO_PROFILE_PDFA_1B,		// PDF/A-1b:2005
+  _PDFIO_PROFILE_PDFA_2A,		// PDF/A-2a:2011
+  _PDFIO_PROFILE_PDFA_2B,		// PDF/A-2b:2011
+  _PDFIO_PROFILE_PDFA_2U,		// PDF/A-2u:2011
+  _PDFIO_PROFILE_PDFA_3A,		// PDF/A-3a:2012
+  _PDFIO_PROFILE_PDFA_3B,		// PDF/A-3b:2012
+  _PDFIO_PROFILE_PDFA_3U,		// PDF/A-3u:2012
+  _PDFIO_PROFILE_PDFA_4			// PDF/A-4:2020
+} _pdfio_profile_t;
+
 typedef ssize_t (*_pdfio_tconsume_cb_t)(void *data, size_t bytes);
 typedef ssize_t (*_pdfio_tpeek_cb_t)(void *data, void *buffer, size_t bytes);
 
@@ -220,15 +236,25 @@ struct _pdfio_dict_s			// Dictionary
 typedef struct _pdfio_objmap_s		// PDF object map
 {
   pdfio_obj_t	*obj;			// Object for this file
-  pdfio_file_t	*src_pdf;		// Source PDF file
+  unsigned char	src_id[32];		// Source PDF file file identifier
   size_t	src_number;		// Source object number
 } _pdfio_objmap_t;
 
+typedef struct _pdfio_strbuf_s		// PDF string buffer
+{
+  struct _pdfio_strbuf_s *next;		// Next string buffer
+  bool		bufused;		// Is this string buffer being used?
+  char		buffer[PDFIO_MAX_STRING + 32];
+					// String buffer
+} _pdfio_strbuf_t;
+
 struct _pdfio_file_s			// PDF file structure
 {
   char		*filename;		// Filename
+  unsigned char	file_id[32];		// File identifier bytes
   struct lconv	*loc;			// Locale data
   char		*version;		// Version number
+  _pdfio_profile_t profile;		// PDF profile, if any
   pdfio_rect_t	media_box,		// Default MediaBox value
 		crop_box;		// Default CropBox value
   _pdfio_mode_t	mode;			// Read/write mode
@@ -257,10 +283,14 @@ struct _pdfio_file_s			// PDF file structure
   pdfio_obj_t	*root_obj;		// Root object/dictionary
   pdfio_obj_t	*info_obj;		// Information object
   pdfio_obj_t	*pages_obj;		// Root pages object
-  pdfio_obj_t	*encrypt_obj;		// De/Encryption object/dictionary
-  pdfio_obj_t	*cp1252_obj,		// CP1252 font encoding object
+  pdfio_obj_t	*encrypt_obj;		// Encryption object (not used for reading)
+  pdfio_dict_t	*encrypt_dict;		// De/Encryption dictionary
+  pdfio_obj_t	*cgats001_obj,		// CGATS001 ICC profile object
+		*cp1252_obj,		// CP1252 font encoding object
 		*unicode_obj;		// Unicode font encoding object
   pdfio_array_t	*id_array;		// ID array
+  bool		encrypt_metadata;	// Encrypt metadata?
+  pdfio_dict_t	*markinfo;		// MarkInfo dictionary, if any
 
   // Allocated data elements
   size_t	num_arrays,		// Number of arrays
@@ -283,6 +313,7 @@ struct _pdfio_file_s			// PDF file structure
   size_t	num_strings,		// Number of strings
 		alloc_strings;		// Allocated strings
   char		**strings;		// Nul-terminated strings
+  _pdfio_strbuf_t *strbuffers;		// String buffers
 };
 
 struct _pdfio_obj_s			// Object
@@ -329,6 +360,7 @@ struct _pdfio_stream_s			// Stream
 
 extern size_t		_pdfio_strlcpy(char *dst, const char *src, size_t dstsize) _PDFIO_INTERNAL;
 extern double		_pdfio_strtod(pdfio_file_t *pdf, const char *s) _PDFIO_INTERNAL;
+extern void		_pdfio_utf16cpy(char *dst, const unsigned char *src, size_t srclen, size_t dstsize) _PDFIO_INTERNAL;
 extern ssize_t		_pdfio_vsnprintf(pdfio_file_t *pdf, char *buffer, size_t bufsize, const char *format, va_list ap) _PDFIO_INTERNAL;
 
 extern bool		_pdfioArrayDecrypt(pdfio_file_t *pdf, pdfio_obj_t *obj, pdfio_array_t *a, size_t depth) _PDFIO_INTERNAL;
@@ -372,7 +404,7 @@ extern bool		_pdfioFileError(pdfio_file_t *pdf, const char *format, ...) _PDFIO_
 extern pdfio_obj_t	*_pdfioFileFindMappedObj(pdfio_file_t *pdf, pdfio_file_t *src_pdf, size_t src_number) _PDFIO_INTERNAL;
 extern bool		_pdfioFileFlush(pdfio_file_t *pdf) _PDFIO_INTERNAL;
 extern int		_pdfioFileGetChar(pdfio_file_t *pdf) _PDFIO_INTERNAL;
-extern bool		_pdfioFileGets(pdfio_file_t *pdf, char *buffer, size_t bufsize) _PDFIO_INTERNAL;
+extern bool		_pdfioFileGets(pdfio_file_t *pdf, char *buffer, size_t bufsize, bool discard) _PDFIO_INTERNAL;
 extern ssize_t		_pdfioFilePeek(pdfio_file_t *pdf, void *buffer, size_t bytes) _PDFIO_INTERNAL;
 extern bool		_pdfioFilePrintf(pdfio_file_t *pdf, const char *format, ...) _PDFIO_INTERNAL;
 extern bool		_pdfioFilePuts(pdfio_file_t *pdf, const char *s) _PDFIO_INTERNAL;
@@ -390,6 +422,8 @@ extern bool		_pdfioObjWriteHeader(pdfio_obj_t *obj) _PDFIO_INTERNAL;
 extern pdfio_stream_t	*_pdfioStreamCreate(pdfio_obj_t *obj, pdfio_obj_t *length_obj, size_t cbsize, pdfio_filter_t compression) _PDFIO_INTERNAL;
 extern pdfio_stream_t	*_pdfioStreamOpen(pdfio_obj_t *obj, bool decode) _PDFIO_INTERNAL;
 
+extern char		*_pdfioStringAllocBuffer(pdfio_file_t *pdf);
+extern void		_pdfioStringFreeBuffer(pdfio_file_t *pdf, char *buffer);
 extern bool		_pdfioStringIsAllocated(pdfio_file_t *pdf, const char *s) _PDFIO_INTERNAL;
 
 extern void		_pdfioTokenClear(_pdfio_token_t *tb) _PDFIO_INTERNAL;
