@@ -259,7 +259,7 @@ _pdfioStreamCreate(
     {
       colors = 1;
     }
-    else if (colors < 0 || colors > 4)
+    else if (colors < 0 || colors > 32)
     {
       _pdfioFileError(st->pdf, "Unsupported Colors value %d.", colors);
       free(st);
@@ -270,7 +270,7 @@ _pdfioStreamCreate(
     {
       columns = 1;
     }
-    else if (columns < 0)
+    else if (columns < 0 || columns > 65536)
     {
       _pdfioFileError(st->pdf, "Unsupported Columns value %d.", columns);
       free(st);
@@ -427,7 +427,7 @@ _pdfioStreamOpen(pdfio_obj_t *obj,	// I - Object
   const char		*type;		// Object type
 
 
-  PDFIO_DEBUG("_pdfioStreamOpen(obj=%p(%u), decode=%s)\n", obj, (unsigned)obj->number, decode ? "true" : "false");
+  PDFIO_DEBUG("_pdfioStreamOpen(obj=%p(%u), decode=%s)\n", (void *)obj, (unsigned)obj->number, decode ? "true" : "false");
 
   // Allocate a new stream object...
   if ((st = (pdfio_stream_t *)calloc(1, sizeof(pdfio_stream_t))) == NULL)
@@ -532,7 +532,7 @@ _pdfioStreamOpen(pdfio_obj_t *obj,	// I - Object
       {
         colors = 1;
       }
-      else if (colors < 0 || colors > 4)
+      else if (colors < 0 || colors > 32)
       {
         _pdfioFileError(st->pdf, "Unsupported Colors value %d.", colors);
 	goto error;
@@ -542,7 +542,7 @@ _pdfioStreamOpen(pdfio_obj_t *obj,	// I - Object
       {
         columns = 1;
       }
-      else if (columns < 0)
+      else if (columns < 0 || columns > 65536)
       {
         _pdfioFileError(st->pdf, "Unsupported Columns value %d.", columns);
 	goto error;
@@ -562,6 +562,13 @@ _pdfioStreamOpen(pdfio_obj_t *obj,	// I - Object
         if (predictor >= 10)
           st->pbsize ++;		// Add PNG predictor byte
 
+        if (st->pbsize < 2)
+        {
+          _pdfioFileError(st->pdf, "Bad Predictor buffer size %lu.", (unsigned long)st->pbsize);
+	  goto error;
+        }
+
+        PDFIO_DEBUG("_pdfioStreamOpen: st->predictor=%d, st->pbpixel=%u, st->pbsize=%lu\n", st->predictor, (unsigned)st->pbpixel, (unsigned long)st->pbsize);
         if ((st->prbuffer = calloc(1, st->pbsize - 1)) == NULL || (st->psbuffer = calloc(1, st->pbsize)) == NULL)
         {
           _pdfioFileError(st->pdf, "Unable to allocate %lu bytes for Predictor buffers.", (unsigned long)st->pbsize);
@@ -616,7 +623,7 @@ _pdfioStreamOpen(pdfio_obj_t *obj,	// I - Object
     else
     {
       // Something else we don't support
-      _pdfioFileError(st->pdf, "Unsupported stream filter '%N'.", filter);
+      _pdfioFileError(st->pdf, "Unsupported stream filter '/%s'.", filter);
       goto error;
     }
   }
@@ -690,8 +697,9 @@ pdfioStreamPeek(pdfio_stream_t *st,	// I - Stream
 // 'pdfioStreamPrintf()' - Write a formatted string to a stream.
 //
 // This function writes a formatted string to a stream.  In addition to the
-// standard `printf` format characters, you can use "%N" to format a PDF name
-// value ("/Name") and "%S" to format a PDF string ("(String)") value.
+// standard `printf` format characters, you can use "%H" to format a HTML/XML
+// string value, "%N" to format a PDF name value ("/Name"), and "%S" to format
+// a PDF string ("(String)") value.
 //
 
 bool					// O - `true` on success, `false` on failure
@@ -840,7 +848,7 @@ pdfioStreamWrite(
 			*pptr;		// Previous raw buffer
 
 
-  PDFIO_DEBUG("pdfioStreamWrite(st=%p, buffer=%p, bytes=%lu)\n", st, buffer, (unsigned long)bytes);
+  PDFIO_DEBUG("pdfioStreamWrite(st=%p, buffer=%p, bytes=%lu)\n", (void *)st, buffer, (unsigned long)bytes);
 
   // Range check input...
   if (!st || st->pdf->mode != _PDFIO_MODE_WRITE || !buffer || !bytes)
@@ -1227,7 +1235,18 @@ stream_read(pdfio_stream_t *st,		// I - Stream
       }
 
       // Apply predictor for this line
-      PDFIO_DEBUG("stream_read: Line %02X %02X %02X %02X %02X.\n", sptr[-1], sptr[0], sptr[0], sptr[2], sptr[3]);
+#ifdef DEBUG
+      if (remaining > 4)
+	PDFIO_DEBUG("stream_read: Line %02X %02X %02X %02X %02X ...\n", sptr[-1], sptr[0], sptr[1], sptr[2], sptr[3]);
+      else if (remaining > 3)
+	PDFIO_DEBUG("stream_read: Line %02X %02X %02X %02X %02X.\n", sptr[-1], sptr[0], sptr[1], sptr[2], sptr[3]);
+      else if (remaining > 2)
+	PDFIO_DEBUG("stream_read: Line %02X %02X %02X %02X.\n", sptr[-1], sptr[0], sptr[1], sptr[2]);
+      else if (remaining > 1)
+	PDFIO_DEBUG("stream_read: Line %02X %02X %02X.\n", sptr[-1], sptr[0], sptr[1]);
+      else
+	PDFIO_DEBUG("stream_read: Line %02X %02X.\n", sptr[-1], sptr[0]);
+#endif // DEBUG
 
       switch (sptr[-1])
       {
